
#pragma once

#include <AK/NonnullRefPtr.h>
#include <AK/StringView.h>
#include <AK/Traits.h>
#include <LibJS/Forward.h>
#include <LibWeb/Forward.h>

namespace Web::CSS {

enum class PropertyID {
    Invalid,
    Custom,
    All,

        Font,

        ListStyle,

        Animation,

        Background,

        BackgroundPosition,

        Border,

        BorderBottom,

        BorderColor,

        BorderLeft,

        BorderRadius,

        BorderRight,

        BorderStyle,

        BorderTop,

        BorderWidth,

        Columns,

        Flex,

        FlexFlow,

        Gap,

        Grid,

        GridArea,

        GridColumn,

        GridGap,

        GridRow,

        GridTemplate,

        Inset,

        InsetBlock,

        InsetInline,

        Margin,

        MarginBlock,

        MarginInline,

        Outline,

        Overflow,

        Padding,

        PaddingBlock,

        PaddingInline,

        PlaceContent,

        PlaceItems,

        PlaceSelf,

        TextDecoration,

        Transition,

        WebkitTextFillColor,

        AccentColor,

        BorderCollapse,

        BorderSpacing,

        CaptionSide,

        ClipRule,

        Color,

        Cursor,

        Direction,

        Fill,

        FillOpacity,

        FillRule,

        FontFamily,

        FontLanguageOverride,

        FontSize,

        FontStyle,

        FontVariant,

        FontWeight,

        FontWidth,

        ImageRendering,

        LetterSpacing,

        LineHeight,

        ListStyleImage,

        ListStylePosition,

        ListStyleType,

        MathDepth,

        MathShift,

        MathStyle,

        PointerEvents,

        Quotes,

        Stroke,

        StrokeOpacity,

        StrokeWidth,

        TextAlign,

        TextAnchor,

        TextDecorationLine,

        TextIndent,

        TextJustify,

        TextShadow,

        TextTransform,

        Visibility,

        WhiteSpace,

        WordSpacing,

        WordWrap,

        AlignContent,

        AlignItems,

        AlignSelf,

        AnimationDelay,

        AnimationDirection,

        AnimationDuration,

        AnimationFillMode,

        AnimationIterationCount,

        AnimationName,

        AnimationPlayState,

        AnimationTimingFunction,

        Appearance,

        AspectRatio,

        BackdropFilter,

        BackgroundAttachment,

        BackgroundClip,

        BackgroundColor,

        BackgroundImage,

        BackgroundOrigin,

        BackgroundPositionX,

        BackgroundPositionY,

        BackgroundRepeat,

        BackgroundSize,

        BorderBottomColor,

        BorderBottomLeftRadius,

        BorderBottomRightRadius,

        BorderBottomStyle,

        BorderBottomWidth,

        BorderLeftColor,

        BorderLeftStyle,

        BorderLeftWidth,

        BorderRightColor,

        BorderRightStyle,

        BorderRightWidth,

        BorderTopColor,

        BorderTopLeftRadius,

        BorderTopRightRadius,

        BorderTopStyle,

        BorderTopWidth,

        Bottom,

        BoxShadow,

        BoxSizing,

        Clear,

        Clip,

        ClipPath,

        ColumnCount,

        ColumnGap,

        ColumnSpan,

        ColumnWidth,

        Content,

        ContentVisibility,

        CounterIncrement,

        CounterReset,

        CounterSet,

        Cx,

        Cy,

        Display,

        FlexBasis,

        FlexDirection,

        FlexGrow,

        FlexShrink,

        FlexWrap,

        Float,

        GridAutoColumns,

        GridAutoFlow,

        GridAutoRows,

        GridColumnEnd,

        GridColumnGap,

        GridColumnStart,

        GridRowEnd,

        GridRowGap,

        GridRowStart,

        GridTemplateAreas,

        GridTemplateColumns,

        GridTemplateRows,

        Height,

        InlineSize,

        InsetBlockEnd,

        InsetBlockStart,

        InsetInlineEnd,

        InsetInlineStart,

        JustifyContent,

        JustifyItems,

        JustifySelf,

        Left,

        MarginBlockEnd,

        MarginBlockStart,

        MarginBottom,

        MarginInlineEnd,

        MarginInlineStart,

        MarginLeft,

        MarginRight,

        MarginTop,

        Mask,

        MaskType,

        MaxHeight,

        MaxInlineSize,

        MaxWidth,

        MinHeight,

        MinInlineSize,

        MinWidth,

        ObjectFit,

        ObjectPosition,

        Opacity,

        Order,

        OutlineColor,

        OutlineOffset,

        OutlineStyle,

        OutlineWidth,

        OverflowX,

        OverflowY,

        PaddingBlockEnd,

        PaddingBlockStart,

        PaddingBottom,

        PaddingInlineEnd,

        PaddingInlineStart,

        PaddingLeft,

        PaddingRight,

        PaddingTop,

        Position,

        R,

        Right,

        RowGap,

        Rx,

        Ry,

        ScrollbarGutter,

        ScrollbarWidth,

        StopColor,

        StopOpacity,

        TableLayout,

        TextDecorationColor,

        TextDecorationStyle,

        TextDecorationThickness,

        TextOverflow,

        Top,

        Transform,

        TransformBox,

        TransformOrigin,

        TransitionDelay,

        TransitionDuration,

        TransitionProperty,

        TransitionTimingFunction,

        UserSelect,

        VerticalAlign,

        Width,

        X,

        Y,

        ZIndex,

};

enum class AnimationType {
    Discrete,
    ByComputedValue,
    RepeatableList,
    Custom,
    None,
};
AnimationType animation_type_from_longhand_property(PropertyID);
bool is_animatable_property(PropertyID);

Optional<PropertyID> property_id_from_camel_case_string(StringView);
Optional<PropertyID> property_id_from_string(StringView);
[[nodiscard]] FlyString const& string_from_property_id(PropertyID);
[[nodiscard]] FlyString const& camel_case_string_from_property_id(PropertyID);
bool is_inherited_property(PropertyID);
NonnullRefPtr<CSSStyleValue> property_initial_value(JS::Realm&, PropertyID);

enum class ValueType {
    Angle,
    BackgroundPosition,
    BasicShape,
    Color,
    Counter,
    CustomIdent,
    EasingFunction,
    FilterValueList,
    Flex,
    Frequency,
    Image,
    Integer,
    Length,
    Number,
    Paint,
    Percentage,
    Position,
    Ratio,
    Rect,
    Resolution,
    String,
    Time,
    Url,
};
bool property_accepts_type(PropertyID, ValueType);
bool property_accepts_keyword(PropertyID, Keyword);
Optional<ValueType> property_resolves_percentages_relative_to(PropertyID);

// These perform range-checking, but are also safe to call with properties that don't accept that type. (They'll just return false.)
bool property_accepts_angle(PropertyID, Angle const&);
bool property_accepts_flex(PropertyID, Flex const&);
bool property_accepts_frequency(PropertyID, Frequency const&);
bool property_accepts_integer(PropertyID, i64 const&);
bool property_accepts_length(PropertyID, Length const&);
bool property_accepts_number(PropertyID, double const&);
bool property_accepts_percentage(PropertyID, Percentage const&);
bool property_accepts_resolution(PropertyID, Resolution const&);
bool property_accepts_time(PropertyID, Time const&);

bool property_is_shorthand(PropertyID);
Vector<PropertyID> longhands_for_shorthand(PropertyID);

size_t property_maximum_value_count(PropertyID);

bool property_affects_layout(PropertyID);
bool property_affects_stacking_context(PropertyID);

constexpr PropertyID first_property_id = PropertyID::Font;
constexpr PropertyID last_property_id = PropertyID::ZIndex;
constexpr PropertyID first_inherited_shorthand_property_id = PropertyID::Font;
constexpr PropertyID last_inherited_shorthand_property_id = PropertyID::ListStyle;
constexpr PropertyID first_inherited_longhand_property_id = PropertyID::WebkitTextFillColor;
constexpr PropertyID last_inherited_longhand_property_id = PropertyID::WordWrap;
constexpr PropertyID first_longhand_property_id = PropertyID::WebkitTextFillColor;
constexpr PropertyID last_longhand_property_id = PropertyID::ZIndex;

enum class Quirk {
    // https://quirks.spec.whatwg.org/#the-hashless-hex-color-quirk
    HashlessHexColor,
    // https://quirks.spec.whatwg.org/#the-unitless-length-quirk
    UnitlessLength,
};
bool property_has_quirk(PropertyID, Quirk);

} // namespace Web::CSS

namespace AK {
template<>
struct Traits<Web::CSS::PropertyID> : public DefaultTraits<Web::CSS::PropertyID> {
    static unsigned hash(Web::CSS::PropertyID property_id) { return int_hash((unsigned)property_id); }
};
} // namespace AK
